#!/usr/bin/env python

import doctest
import os

def run_tests():
    directory = os.path.dirname(os.path.abspath(__file__))
    tests =  [ test for test in os.listdir(directory) if test.endswith('.txt') ]

    for test in tests:
        doctest.testfile(test)

if __name__ == '__main__':
    run_tests()



"""
a view with webob + genshi for editing and viewing .ini files
"""
import os
import re

from genshi.template import TemplateLoader
from martini.config import ConfigMunger
from martini.utils import getbool
from martini.utils import getlist
from paste.httpexceptions import HTTPExceptionHandler
from paste.urlparser import StaticURLParser
from pkg_resources import resource_filename
from webob import Request, Response, exc

class MartiniWebView(object):

    ### class level variables
    defaults = { 'auto_reload': 'True',
                 'files': None,
                 'directories': None }

    def __init__(self, **kw):

        # set instance parameters from kw and defaults
        for key in self.defaults:
            setattr(self, key, kw.get(key, self.defaults[key]))
        self.auto_reload = getbool(self.auto_reload)

        # files
        self.files = getlist(self.files)
        if self.files:
            assert [ f for f in self.files if os.path.isabs(f) ]
            self.files = dict([(os.path.basename(f), f) 
                               for f in self.files])
        else:
            self.files = {}

        # directories
        self.directories = getlist(self.directories)
        if self.directories:
            assert [ d for d in self.directories if os.path.isabs(d) ]
            if len(self.directories) > 1 or self.files:
                # namespace directories
                self.directories = dict([(os.path.basename(d), d)
                                         for d in self.directories])
            else:
                # don't namespace a single directory
                self.directories = { '': self.directories[0] }
        else:
            self.directories = {}

        # have to have something to serve!
        assert self.files or self.directories

        # methods to respond to
        self.response_functions = { 'GET': self.get,
                                    'POST': self.post,
                                    }

        # template loader
        templates_dir = resource_filename(__name__, 'templates')
        self.loader = TemplateLoader(templates_dir,
                                     auto_reload=self.auto_reload)

        # fileserver
        self.fileserver = StaticURLParser(resource_filename(__name__, 'static'))

    def file(self, path):
        path = path.strip('/')
        if not path:
            return None
        f = self.files.get(path, None)
        if f is None:
            if path in self.directories:
                return self.directories[path]
            if '/' in path:
                path = path.split('/', 1)[-1]
                for d in self.directories.values():
                    filename = os.path.join(d, path)
                    if os.path.exists(filename):
                        return filename
        else:
            return f

    ### methods dealing with HTTP
    def __call__(self, environ, start_response):
        request = Request(environ)

        if request.path_info.strip('/') and os.path.exists(os.path.join(resource_filename(__name__, 'static'), request.path_info.strip('/'))):
            return self.fileserver(environ, start_response)

        if request.path_info.endswith('/'):
            if request.path_info.strip('/'):
                raise exc.HTTPFound(location=request.path_info.rstrip('/'))
            else:
                if request.path_info != '/':
                    raise exc.HTTPFound(location='/')
        res = self.make_response(request)
        return res(environ, start_response)
                                
    def make_response(self, request):
        return self.response_functions.get(request.method, self.error)(request)

    def get_response(self, text, content_type='text/html'):
        """make an HTTP response from text"""
        res = Response(content_type=content_type, body=text)
        return res

    def get(self, request):
        """
        return response to a GET requst
        """

        # index of all resources
        if not request.path_info.strip('/'):
            template = self.loader.load('index.html')
            items = self.files.keys() + self.directories.keys()
            res = template.generate(request=request, items=items).render('html', doctype='html')
            return self.get_response(res)

        # get the file
        f = self.file(request.path_info)
        if not f:
            raise exc.HTTPNotFound()
        
        # index page of a directory
        if os.path.isdir(f):
            template = self.loader.load('index.html')
            items = os.listdir(f)
            res = template.generate(request=request, items=items).render('html', doctype='html')
            return self.get_response(res)

        # get configuration
        conf = ConfigMunger(f)

        # render the template
        template = self.loader.load('table.html')
        res = template.generate(request=request, sections=conf.dict()).render('html', doctype='html')
        # generate the response
        return self.get_response(res)

    def post(self, request):
        """
        return response to a POST request
        """

        if not request.path_info.strip('/'):
            raise exc.HTTPMethodNotAllowed()

        # get the file
        f = self.file(request.path_info)
        if not f:
            raise exc.HTTPNotFound()
        if os.path.isdir(f):
            raise exc.HTTPMethodNotAllowed()

        regex = '\[([^\]]+)\](.*)'

        conf = ConfigMunger(f)

        delete = request.POST.getall('delete')
        if delete:
            del request.POST['delete']

        for key, value in request.POST.items():
            value = ' '.join(value.strip().split())
            match = re.match(regex, key)
            section, option = match.groups()
            if option:
                conf.set(section, option, value)
            else:
                if value:
                    conf.move_section(section, value)
                else:
                    conf.add_section(section)
        
        for d in delete:
            match = re.match(regex, d)
            section, option = match.groups()
            if conf.has_section(section):
                if option:
                    conf.remove_option(section, option)
                else:
                    conf.remove_section(section)

        output = file(f, 'w')
        conf.write(output)

        return exc.HTTPSeeOther(location=request.path_info)

    def error(self, request):
        """deal with non-supported methods"""
        return exc.HTTPMethodNotAllowed("Only %r operations are allowed" % self.response_functions.keys())


### paste factory
        
def factory(global_conf, **app_conf):
    """create a webob view and wrap it in middleware"""
    keystr = 'martini.'
    args = dict([(key.split(keystr, 1)[-1], value)
                 for key, value in app_conf.items()
                 if key.startswith(keystr) ])
    app = MartiniWebView(**args)
    return HTTPExceptionHandler(app)



from setuptools import setup, find_packages
import sys, os

try:
    description = file('README.txt').read()
except:
    description = ''

version = '0.3.2'

setup(name='martINI',
      version=version,
      description="edit .ini files from the command line",
      long_description=description,
      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='ini cli',
      author='Jeff Hammel',
      author_email='k0scist@gmail.com',
      url='http://k0s.org/hg/martINI',
      license='GPL',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
         'WebOb',	
         'Paste',
         'PasteScript',
         'genshi'
      ],
      entry_points="""
      # -*- Entry points: -*-
      [console_scripts]
      ini-get = martini.main:get
      ini-set = martini.main:set
      ini-delete = martini.main:delete
      ini-munge = martini.main:munge

      [paste.app_factory]
      main = martini.web:factory
      """,
      )



#!/usr/bin/env python

import os
import sys
import urllib2

from ConfigParser import ConfigParser
from ConfigParser import InterpolationMissingOptionError
from ConfigParser import MissingSectionHeaderError
from ConfigParser import NoOptionError
from StringIO import StringIO

def file_pointer(resource):
    """returns a file-like object given a string"""
    # XXX could go in utils.py 

    if not isinstance(resource, basestring):
        # assume resource is already a file-like object
        return resource

    if os.path.exists(resource):
        return file(resource)
    if sum([resource.startswith(http) for http in 'http://', 'https://']):
        return urllib2.urlopen(resource)
    return StringIO(resource)


class ConfigMunger(ConfigParser):
    """combine configuration from .ini files"""
    
    def __init__(self, *conf, **kw):
        ConfigParser.__init__(self, kw.get('defaults',{}))
        self.read(*conf)
    
    def __getitem__(self, section):
        """
        return an object with __getitem__ defined appropriately
        to allow referencing like self['foo']['bar']
        """
        return dict(self.items(section))

    def get(self, section, option, default=None, raw=False, vars=None):
        try:
            value = ConfigParser.get(self, section, option, raw, vars)
        except NoOptionError:
            return default
        return value

    def set(self, section, option, value):
        if section not in self.sections():
            self.add_section(section)
        ConfigParser.set(self, section, option, value)

    def move_section(self, section, newname):
        if self.has_section(section):
            _section = self[section]
            self.remove_section(section)
        else:
            _section = {}
        self.read({newname: _section})
    
    def dict(self):
        """return a dictionary of dictionaries; 
        the outer with keys of section names;
        the inner with keys, values of the section"""
        return dict([(section, self[section])
                     for section in self.sections()])

    def read(self, *ini):
        for _ini in ini:
            if isinstance(_ini, dict):
                for section, contents in _ini.items():
                    for option, value in contents.items():
                        self.set(section, option, value)
            elif isinstance(_ini, list) or isinstance(_ini, tuple):

                # ensure list or tuple of 3-tuples
                assert len([option for option in _ini
                            if isinstance(option, tuple) 
                            and len(option) == 3])

                for section, option, value in _ini:
                    self.set(section, option, value)                
            else:
                fp = file_pointer(_ini)
                try:
                    self.readfp(fp)
                except MissingSectionHeaderError:
                    fp.seek(0)
                    fp = StringIO("[DEFAULTS]\n" + fp.read())
                    self.readfp(fp)
            
    def missing(self):
        """returns missing variable names"""
        missing = set()        

        for section in self.sections():
            for key, val in self.items(section, raw=True):
                try:
                    self.get(section, key)
                except InterpolationMissingOptionError, e:
                    missing.add(e.reference)
        return missing

    def tuples(self):
        """
        return options in format appropriate to trac:
        [ (section, option, value) ]
        """
        options = []
        for section in self.sections():
            options.extend([(section,) + item 
                            for item in self.items(section)])
        return options

    def write(self, fp=sys.stdout, raw=False, sorted=True, vars=None):
        sections = self.sections()
        if sorted:
            sections.sort()

        for section in sections:
            print >> fp, '[%s]' % section
            options = self.options(section)
            if sorted:
                options.sort()
            for option in options:
                print >> fp, "%s = %s" % (option, self.get(section, option, raw, vars))
            if section != sections[-1]:
                print >> fp

if __name__ == '__main__':
    import sys
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option('--missing', action="store_true", default=False,
                      help="list missing template variables")
    munger = ConfigMunger()
    options, args = parser.parse_args()
    munger.read(*args)
    if options.missing:
        for missing in munger.missing():
            print missing
    else:
        munger.write(sys.stdout)
